@@ -2,10 +2,10 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.6.0", "1.6.0")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "i686", "2010-03-31 14:09:15 EDT")]
+[assembly: ApplicationVersion ("1.7.0", "1.7.0")]
+[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2010-05-05 13:05:48 PDT")]
 
-[assembly: AssemblyVersion ("1.6.0.0")]
+[assembly: AssemblyVersion ("1.7.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
 [assembly: AssemblyDescription ("Banshee Media Player")]
 [assembly: AssemblyCopyright ("Copyright (C) 2005-2010 Novell Inc. and others")]
