@@ -124,11 +124,12 @@ namespace Banshee.Streaming
         public static void TrackInfoMerge (TrackInfo track, SafeUri uri)
         {
             track.Uri = uri;
-            TagLib.File file = StreamTagger.ProcessUri (uri);
-            TrackInfoMerge (track, file);
+            using (var file = StreamTagger.ProcessUri (uri)) {
+                TrackInfoMerge (track, file);
 
-            if (file == null) {
-                track.TrackTitle = uri.AbsoluteUri;
+                if (file == null) {
+                    track.TrackTitle = uri.AbsoluteUri;
+                }
             }
         }
 
@@ -321,6 +322,7 @@ namespace Banshee.Streaming
             }
 
             file.Save ();
+            file.Dispose ();
 
             track.FileSize = Banshee.IO.File.GetSize (track.Uri);
             track.FileModifiedStamp = Banshee.IO.File.GetModifiedTime (track.Uri);
